@@ -1,9 +1,10 @@
+from __future__ import print_function
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 ''' Post installation script for linux '''
 
-import sys, os, cPickle, textwrap, stat, errno
+import sys, os, textwrap, stat, errno
 from subprocess import check_call, check_output
 from functools import partial
 
@@ -22,11 +23,11 @@ entry_points = {
              'ebook-polish         = calibre.ebooks.oeb.polish.main:main',
              'markdown-calibre     = calibre.ebooks.markdown.__main__:run',
              'web2disk             = calibre.web.fetch.simple:main',
-             'calibre-server       = calibre.library.server.main:main',
+             'calibre-server       = calibre.srv.standalone:main',
              'lrf2lrs              = calibre.ebooks.lrf.lrfparser:main',
              'lrs2lrf              = calibre.ebooks.lrf.lrs.convert_from:main',
              'calibre-debug        = calibre.debug:main',
-             'calibredb            = calibre.library.cli:main',
+             'calibredb            = calibre.db.cli.main:main',
              'calibre-parallel     = calibre.utils.ipc.worker:main',
              'calibre-customize    = calibre.customize.ui:main',
              'calibre-complete     = calibre.utils.complete:main',
@@ -329,7 +330,7 @@ class ZshCompleter(object):  # {{{
                                 '--input-encoding' not in o._long_opts]
                 if not opts:
                     continue
-                opts = '\\\n  '.join(tuple(self.get_options(opts)))
+                opts = '\\\n  '.join(tuple(sorted(self.get_options(opts))))
                 w('\n%s() {'%(func%fmt))
                 w('\n  _arguments -s \\\n  ' + opts)
                 w('\n}\n')
@@ -372,27 +373,27 @@ class ZshCompleter(object):  # {{{
             opt_lines.append(ostrings + help_txt + ' \\')
         opt_lines = ('\n' + (' ' * 8)).join(opt_lines)
 
-        f.write((ur'''
+        f.write((u'''
 _ebook_edit() {
     local curcontext="$curcontext" state line ebookfile expl
     typeset -A opt_args
 
-    _arguments -C -s \
+    _arguments -C -s \\
         %s
-        "1:ebook file:_files -g '(#i)*.(%s)'" \
+        "1:ebook file:_files -g '(#i)*.(%s)'" \\
         '*:file in ebook:->files' && return 0
 
     case $state in
         files)
             ebookfile=${~${(Q)line[1]}}
 
-            if [[ -f "$ebookfile" && "$ebookfile" =~ '\.[eE][pP][uU][bB]$' ]]; then
+            if [[ -f "$ebookfile" && "$ebookfile" =~ '\\.[eE][pP][uU][bB]$' ]]; then
                 _zip_cache_name="$ebookfile"
                 _zip_cache_list=( ${(f)"$(zipinfo -1 $_zip_cache_name 2>/dev/null)"} )
             else
                 return 1
             fi
-            _wanted files expl 'file from ebook' \
+            _wanted files expl 'file from ebook' \\
             _multi_parts / _zip_cache_list && return 0
             ;;
     esac
@@ -402,15 +403,11 @@ _ebook_edit() {
 ''' % (opt_lines, '|'.join(tweakable_fmts)) + '\n\n').encode('utf-8'))
 
     def do_calibredb(self, f):
-        import calibre.library.cli as cli
+        from calibre.db.cli.main import COMMANDS, option_parser_for
         from calibre.customize.ui import available_catalog_formats
         parsers, descs = {}, {}
-        for command in cli.COMMANDS:
-            op = getattr(cli, '%s_option_parser'%command)
-            args = [['t.epub']] if command == 'catalog' else []
-            p = op(*args)
-            if isinstance(p, tuple):
-                p = p[0]
+        for command in COMMANDS:
+            p = option_parser_for(command)()
             parsers[command] = p
             lines = [x.strip().partition('.')[0] for x in p.usage.splitlines() if x.strip() and
                      not x.strip().startswith('%prog')]
@@ -515,7 +512,7 @@ def write_completion(bash_comp_dest, zsh):
     from calibre.ebooks.metadata.sources.cli import option_parser as fem_op
     from calibre.gui2.main import option_parser as guiop
     from calibre.utils.smtp import option_parser as smtp_op
-    from calibre.library.server.main import option_parser as serv_op
+    from calibre.srv.standalone import create_option_parser as serv_op
     from calibre.ebooks.oeb.polish.main import option_parser as polish_op, SUPPORTED
     from calibre.ebooks.oeb.polish.import_book import IMPORTABLE
     from calibre.debug import option_parser as debug_op
@@ -648,12 +645,12 @@ class PostInstall:
         import traceback
         tb = '\n\t'.join(traceback.format_exc().splitlines())
         self.info('\t'+tb)
-        print
+        print()
 
     def warning(self, *args, **kwargs):
-        print '\n'+'_'*20, 'WARNING','_'*20
+        print('\n'+'_'*20, 'WARNING','_'*20)
         prints(*args, **kwargs)
-        print '_'*50
+        print('_'*50)
         print ('\n')
         self.warnings.append((args, kwargs))
         sys.stdout.flush()
@@ -675,7 +672,8 @@ class PostInstall:
         self.opts.staging_etc = '/etc' if self.opts.staging_root == '/usr' else \
                 os.path.join(self.opts.staging_root, 'etc')
 
-        scripts = cPickle.loads(P('scripts.pickle', data=True))
+        from calibre.utils.serialize import msgpack_loads
+        scripts = msgpack_loads(P('scripts.calibre_msgpack', data=True))
         self.manifest = manifest or []
         if getattr(sys, 'frozen_path', False):
             if os.access(self.opts.staging_bindir, os.W_OK):
@@ -707,7 +705,7 @@ class PostInstall:
             self.info('\n\nThere were %d warnings\n'%len(self.warnings))
             for args, kwargs in self.warnings:
                 self.info('*', *args, **kwargs)
-                print
+                print()
 
     def create_uninstaller(self):
         base = self.opts.staging_bindir
@@ -745,7 +743,7 @@ class PostInstall:
             write_completion(bash_comp_dest, zsh)
         except TypeError as err:
             if 'resolve_entities' in str(err):
-                print 'You need python-lxml >= 2.0.5 for calibre'
+                print('You need python-lxml >= 2.0.5 for calibre')
                 sys.exit(1)
             raise
         except EnvironmentError as e:
@@ -833,7 +831,7 @@ class PostInstall:
                 f.close()
                 des = ('calibre-gui.desktop', 'calibre-lrfviewer.desktop',
                         'calibre-ebook-viewer.desktop', 'calibre-ebook-edit.desktop')
-                appdata = os.path.join(os.path.dirname(self.opts.staging_sharedir), 'appdata')
+                appdata = os.path.join(os.path.dirname(self.opts.staging_sharedir), 'metainfo')
                 if not os.path.exists(appdata):
                     try:
                         os.mkdir(appdata)
@@ -922,15 +920,17 @@ complete -F _'''%(opts, words) + fname + ' ' + name +"\n\n").encode('utf-8')
 
 
 pics = {'jpg', 'jpeg', 'gif', 'png', 'bmp'}
+pics = list(sorted(pics))  # for reproducability
 
 
 def opts_and_exts(name, op, exts, cover_opts=('--cover',), opf_opts=(),
                   file_map={}):
     opts = ' '.join(options(op))
     exts.extend([i.upper() for i in exts])
-    exts='|'.join(exts)
+    exts='|'.join(sorted(exts))
     fname = name.replace('-', '_')
-    spics = '|'.join(tuple(pics) + tuple(x.upper() for x in pics))
+    spics = pics + [i.upper() for i in pics]
+    spics = '|'.join(sorted(spics))
     special_exts_template = '''\
       %s )
            _filedir %s
@@ -940,7 +940,7 @@ def opts_and_exts(name, op, exts, cover_opts=('--cover',), opf_opts=(),
     extras = []
     for eopts, eexts in ((cover_opts, "${pics}"), (opf_opts, "'@(opf)'")):
         for opt in eopts:
-            extras.append(special_exts_template%(opt, sorted(eexts)))
+            extras.append(special_exts_template%(opt, eexts))
     extras = '\n'.join(extras)
 
     return '_'+fname+'()'+\
@@ -971,7 +971,7 @@ def opts_and_exts(name, op, exts, cover_opts=('--cover',), opf_opts=(),
 
 }
 complete -o filenames -F _'''%dict(pics=spics,
-    opts=opts, extras=extras, exts=sorted(exts)) + fname + ' ' + name +"\n\n"
+    opts=opts, extras=extras, exts=exts) + fname + ' ' + name +"\n\n"
 
 
 VIEWER = '''\
@@ -1005,9 +1005,9 @@ ETWEAK = '''\
 [Desktop Entry]
 Version=1.0
 Type=Application
-Name=Edit E-book
-GenericName=Edit E-books
-Comment=Edit e-books in various formats
+Name=E-book Editor
+GenericName=Editor for E-books
+Comment=Edit E-books in various formats
 TryExec=ebook-edit
 Exec=ebook-edit --detach %f
 Icon=calibre-ebook-edit
@@ -1080,7 +1080,7 @@ def write_appdata(key, entry, base, translators):
     fpath = os.path.join(base, '%s.appdata.xml' % key)
     screenshots = E.screenshots()
     for w, h, url in entry['screenshots']:
-        s = E.screenshot(url, width=str(w), height=str(h))
+        s = E.screenshot(E.image(url, width=str(w), height=str(h)))
         screenshots.append(s)
     screenshots[0].set('type', 'default')
     description = E.description()
@@ -1092,8 +1092,8 @@ def write_appdata(key, entry, base, translators):
                 description.append(E.p(tp))
                 description[-1].set('{http://www.w3.org/XML/1998/namespace}lang', lang)
 
-    root = E.application(
-        E.id(key + '.desktop', type='desktop'),
+    root = E.component(
+        E.id(key + '.desktop'),
         E.name(entry['name']),
         E.metadata_license('CC0-1.0'),
         E.project_license('GPL-3.0'),
@@ -1101,6 +1101,7 @@ def write_appdata(key, entry, base, translators):
         description,
         E.url('https://calibre-ebook.com', type='homepage'),
         screenshots,
+        type='desktop'
     )
     for lang, t in translators.iteritems():
         tp = t.ugettext(entry['summary'])
@@ -1127,12 +1128,12 @@ def main():
 
 def cli_index_strings():
     return _('Command Line Interface'), _(
-        'On OS X, the command line tools are inside the calibre bundle, for example,'
+        'On macOS, the command line tools are inside the calibre bundle, for example,'
     ' if you installed calibre in :file:`/Applications` the command line tools'
     ' are in :file:`/Applications/calibre.app/Contents/console.app/Contents/MacOS/`.'), _(
         'Documented commands'), _('Undocumented commands'), _(
         'You can see usage for undocumented commands by executing them without arguments in a terminal.'), _(
-            'Change language')
+            'Change language'), _('Search')
 
 
 if __name__ == '__main__':
